# Last Modified: 1999:06:10 15:18:32

#from generic import GenericScanner
#from token import Token
from flconvert import *

import re


# This is a list of data file keywords.  Nodes that match here will
# be given a type equal to the match (ie: node.type="header_name".
# Otherwise, the node will be of type 'identifier' with a value
# equal to the string.
keywordRegex = "header_name|code_name|snap|version|grid[xy]|Function|" + \
"return_type|const|" + \
"do_not_include_H_from_C|callback|user_data|class|xywh|box|code[0-3]?|when|" + \
"labeltype|labelsize|labelfont|labelcolor|labelalign|label|selection_color|" + \
"color|minimum|maximum|step|objectType|align|open|selected|value|decl|" + \
"down_box|shortcut"

codeDefRegex = re.compile("code[0-4]")


class FlScanner(GenericScanner):

	def __init__(self, keywordRegexString=keywordRegex):
		GenericScanner.__init__(self)
		self.keywordRegex = re.compile(keywordRegexString)

	def t_string(self, s):
		r' "[^}]*" '
		self.rv.append( Token(type="string", attr=s))

	def t_name(self, s):
		r'  [a-zA-Z_] [a-zA-Z0-9_\.]* '
		match = self.keywordRegex.match(s)
		if match and match.group(0) == s:
			if codeDefRegex.match(s):
				self.rv.append(Token(type='codeDefHeader'))
			else:
				self.rv.append(Token(type=s))

		else:
			self.rv.append(Token(type="identifier", attr=s))

	def	tokenize(self, input):
		self.rv = []
		GenericScanner.tokenize(self, input)
		return self.rv

	def t_whitespace(self, s):
		r' \s+ '
		pass


	def t_comment(self, s):
		r' \# [^\r\n]*'
		pass
		#self.rv.append(Token(type="comment"))

	
	def t_punctuationTerminal(self, s):
		r' {|}|\.|\(|\)|\*|\&|\,|\: '
		self.rv.append(Token(type=s))

	def t_number(self, s):
		r' \d+ '
		t = Token(type='number', attr=s)
		self.rv.append(t)

	def t_float(self, s):
		r' \d+ \. \d+ '
		t = Token(type='float', attr=s)
		self.rv.append(t)

	def t_hexadecimal(self, s):
		r' 0x\d+ '
		t = Token(type='hexadecimal', attr=s)
		self.rv.append(t)



	
def scan(f):
	input = f.read()
	scanner = FlScanner()
	return scanner.tokenize(input)


if __name__ == '__main__':
	import sys
	filename = sys.argv[1]
	f = open(filename)
	print scan(f)


